/*
 * File: App.js 
 */

/*jslint node: true, vars: true */
/*global gEngine: false, Scene: false, SpriteRenderable: false, Camera: false, vec2: false,
  TextureRenderable: false, Renderable: false, SpriteAnimateRenderable: false, GameOver: false,
  FontRenderable: false */
/* find out more about jslint: http://www.jslint.com/help.html */

"use strict";  // Operate in Strict mode such that variables must be declared before used!

function App()
{
    // this.kRedFlower = "https://8observer8.github.io/Assets/WebGLProgrammingGuide/resources/redflower.jpg";
    // this.kStar = "https://8observer8.github.io/Assets/MyImages/Stars/Star64.png";

    this.kRedFlower = "assets/images/redflower.jpg";
    this.kStar = "assets/images/Star64.png";

    this.kFontCon16 = "assets/fonts/Consolas-16";  // notice font names do not need extensions!
    this.kFontCon24 = "assets/fonts/Consolas-24";
    this.kFontCon32 = "assets/fonts/Consolas-32";  // this is also the default system font
    this.kFontCon72 = "assets/fonts/Consolas-72";
    this.kFontSeg96 = "assets/fonts/Segment7-96";

    // this.kFontCon16 = "https://8observer8.github.io/Assets/BuildYourOwn2DGameEngine/Chapter05/5.4.FontSupport/font/Consolas-16";
    // this.kFontCon24 = "https://8observer8.github.io/Assets/BuildYourOwn2DGameEngine/Chapter05/5.4.FontSupport/font/Consolas-24";
    // this.kFontCon32 = "https://8observer8.github.io/Assets/BuildYourOwn2DGameEngine/Chapter05/5.4.FontSupport/font/Consolas-32";
    // this.kFontCon72 = "https://8observer8.github.io/Assets/BuildYourOwn2DGameEngine/Chapter05/5.4.FontSupport/font/Consolas-72";
    // this.kFontSeg96 = "https://8observer8.github.io/Assets/BuildYourOwn2DGameEngine/Chapter05/5.4.FontSupport/font/Segment7-96";

    // The camera to view the scene
    this.mCamera = null;

    this.mRedFlower = null;
    this.mRedFlowerFont = null;

    this.mStar = null;
    this.mStarFont = null;

    this.mSquare = null;

    this.mTextSysFont = null;
    this.mTextCon16 = null;
    this.mTextCon24 = null;
    this.mTextCon32 = null;
    this.mTextCon72 = null;
    this.mTextSeg96 = null;
}

gEngine.Core.inheritPrototype(App, Scene);

App.prototype.loadScene = function ()
{
    // Loads the textures
    gEngine.Textures.loadTexture(this.kRedFlower);
    gEngine.Textures.loadTexture(this.kStar);

    // Loads all the fonts
    gEngine.Fonts.loadFont(this.kFontCon16);
    gEngine.Fonts.loadFont(this.kFontCon24);
    gEngine.Fonts.loadFont(this.kFontCon32);
    gEngine.Fonts.loadFont(this.kFontCon72);
    gEngine.Fonts.loadFont(this.kFontSeg96);
};

App.prototype.unloadScene = function ()
{
    // App loop not running, unload all assets

    gEngine.Textures.unloadTexture(this.kRedFlower);
    gEngine.Textures.unloadTexture(this.kStar);

    // unload the fonts
    gEngine.Fonts.unloadFont(this.kFontCon16);
    gEngine.Fonts.unloadFont(this.kFontCon24);
    gEngine.Fonts.unloadFont(this.kFontCon32);
    gEngine.Fonts.unloadFont(this.kFontCon72);
    gEngine.Fonts.unloadFont(this.kFontSeg96);
};

App.prototype.initialize = function ()
{
    // Step A: set up the cameras
    this.mCamera = new Camera(
        vec2.fromValues(50, 33),   // position of the camera
        100,                       // width of camera
        [20, 40, 600, 400]         // viewport (orgX, orgY, width, height)
    );
    this.mCamera.setBackgroundColor([0.8, 0.8, 0.8, 1]);
    // sets the background to gray

    this.mRedFlower = new TextureRenderable(this.kRedFlower);
    this.mRedFlower.setColor([0, 0, 0, 0]);
    this.mRedFlower.getXform().setPosition(25, 30);
    this.mRedFlower.getXform().setSize(30, 30);

    this.mStar = new TextureRenderable(this.kStar);
    this.mStar.setColor([0, 0, 0, 0]);
    this.mStar.getXform().setPosition(60, 10);
    this.mStar.getXform().setSize(10, 10);

    this.mSquare = new Renderable();
    this.mSquare.setColor([0.2, 0.3, 0.2, 1]);
    this.mSquare.getXform().setPosition(10, 10);
    this.mSquare.getXform().setSize(15, 15);

    var redFlowerText = "JPG image (without transparency)";
    this.mRedFlowerFont = new FontRenderable(redFlowerText);
    this._initText(
        this.mRedFlowerFont,
        this.mRedFlower.getXform().getXPos() - ((this.mRedFlowerFont.getXform().getWidth() / 2) * redFlowerText.length),
        this.mRedFlower.getXform().getYPos() + this.mRedFlower.getXform().getHeight() / 2 + this.mRedFlowerFont.getXform().getHeight() * 2,
        [0, 0.5, 0, 1], 3);

    var starText = "PNG image with transparency";
    this.mStarFont = new FontRenderable(starText);
    this._initText(
        this.mStarFont,
        this.mStar.getXform().getXPos() - ((this.mStarFont.getXform().getWidth() / 2) * starText.length),
        this.mStar.getXform().getYPos() - this.mStar.getXform().getHeight() / 2 - this.mStarFont.getXform().getHeight() * 2,
        [0.5, 0, 0, 1], 3);

    this.mTextSysFont = new FontRenderable("System Font: in Red");
    this._initText(this.mTextSysFont, 50, 60, [1, 0, 0, 1], 3);

    this.mTextCon16 = new FontRenderable("Consolas 16: in black");
    this.mTextCon16.setFont(this.kFontCon16);
    this._initText(this.mTextCon16, 50, 55, [0, 0, 0, 1], 2);

    this.mTextCon24 = new FontRenderable("Consolas 24: in black");
    this.mTextCon24.setFont(this.kFontCon24);
    this._initText(this.mTextCon24, 50, 50, [0, 0, 0, 1], 3);

    this.mTextCon32 = new FontRenderable("Consolas 32: in white");
    this.mTextCon32.setFont(this.kFontCon32);
    this._initText(this.mTextCon32, 40, 40, [1, 1, 1, 1], 4);

    this.mTextCon72 = new FontRenderable("Consolas 72: in blue");
    this.mTextCon72.setFont(this.kFontCon72);
    this._initText(this.mTextCon72, 30, 30, [0, 0, 1, 1], 6);

    this.mTextSeg96 = new FontRenderable("Segment7-92");
    this.mTextSeg96.setFont(this.kFontSeg96);
    this._initText(this.mTextSeg96, 30, 15, [1, 1, 0, 1], 7);
    //</editor-fold>
};

App.prototype._initText = function (font, posX, posY, color, textH)
{
    font.setColor(color);
    font.getXform().setPosition(posX, posY);
    font.setTextHeight(textH);
};

// This is the draw function, make sure to setup proper drawing environment, and more
// importantly, make sure to _NOT_ change any state.
App.prototype.draw = function ()
{
    // Step A: clear the canvas
    gEngine.Core.clearCanvas([0.9, 0.9, 0.9, 1.0]); // clear to light gray

    // Step  B: Activate the drawing Camera
    this.mCamera.setupViewProjection();

    this.mRedFlower.draw(this.mCamera.getVPMatrix());
    this.mRedFlowerFont.draw(this.mCamera.getVPMatrix());

    this.mSquare.draw(this.mCamera.getVPMatrix());

    // drawing the text output
    this.mTextSysFont.draw(this.mCamera.getVPMatrix());
    this.mTextCon16.draw(this.mCamera.getVPMatrix());
    this.mTextCon24.draw(this.mCamera.getVPMatrix());
    this.mTextCon32.draw(this.mCamera.getVPMatrix());
    this.mTextCon72.draw(this.mCamera.getVPMatrix());
    this.mTextSeg96.draw(this.mCamera.getVPMatrix());

    this.mStar.draw(this.mCamera.getVPMatrix());
    this.mStarFont.draw(this.mCamera.getVPMatrix());
};

// The function, updates the application state. Make sure to _NOT_ draw
// anything from this function!
App.prototype.update = function ()
{
    var deltaX = 0.5;
    var xformSysFont = this.mTextSysFont.getXform();
    var xformSquare = this.mSquare.getXform();
    var xformStar = this.mStar.getXform();

    xformSysFont.incXPosBy(deltaX);
    if (xformSysFont.getXPos() > 100)
    { // this is the right-bound of the window
        xformSysFont.setPosition(-50, 60);
    }

    xformSquare.incRotationByDegree(1);
    xformStar.incRotationByDegree(-1);
};
